@@ -1,5 +1,5 @@
 /* clock.cxx
-     $Id: clock.cxx,v 1.26 1998/10/26 19:03:44 elf Exp $
+     $Id: clock.cxx,v 1.28 2001/10/31 04:55:54 elf Exp $
 
    written by Marc Singer
    3 April 1997
@@ -49,7 +49,17 @@
 # include <stdarg.h>
 #endif
 #include <math.h>
-#include <sys/time.h>
+
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
 #include "version.h"
 
@@ -86,6 +96,7 @@ bool g_fQuit;
 int do_clock (void);
 void signal_alarm (LSignalHandle handle, void* pv);
 int x_after (Display*);
+time_t calc_delta (time_t);
 
 class WTopLevel : public LWindow {
 protected:
@@ -95,7 +106,6 @@ protected:
 #if defined (USE_FACE_BACKING)
   Pixmap m_pixmapFace;		// X drawable for backing clock face
 #endif
-  time_t m_secondsDeltaZone;	// Timezone correction
   time_t m_seconds;		// Last seconds value rendered
   bool m_fExpose;		// Exposure event forces redraw
 
@@ -210,6 +220,43 @@ EventMap g_rgEMTop[] = {
 
   /* ----- Methods */
 
+/* calc_delta
+
+   calculates/caches the timezone delta.  It performs some caching
+   checks to make sure it isn't calculated needlessly.
+
+   It computes the delta when timeNow
+
+*/
+
+time_t calc_delta (time_t timeNow)
+{
+  static time_t timeLastCalc;	// Time delta was last computed
+  static time_t timeDelta;	// Last computed delta 
+  const int timeRecalc = 60*60;	// Interval for recalc, aligned to 12:00:00am
+
+  if (timeNow == 0)
+    time (&timeNow);
+  if (timeNow != timeLastCalc
+      && (   timeNow - timeLastCalc > timeRecalc
+	  || (timeNow%timeRecalc) == 0)) {
+    //    fprintf (stderr, "recalc delta\n");
+
+				// -- Determine timezone correction
+    struct tm tmGMT   = *gmtime (&timeNow);
+    struct tm tmLocal = *localtime (&timeNow);
+    tmGMT.tm_isdst = 0;		// We want the absolute difference
+    tmLocal.tm_isdst = 0;
+    timeDelta = mktime (&tmLocal) - mktime (&tmGMT);
+  //  fprintf (stderr, "deltaZone is %ld seconds or %d hours\n", 
+  //	   timeDelta, timeDelta/3600);
+    timeLastCalc = timeNow;
+  }
+  
+  return timeDelta;
+}
+
+
 int round_at (double r, double c, bool fInvert = false)
 {
 #if 1
@@ -413,7 +460,7 @@ bool WTopLevel::render (void)
 #else
   time_t seconds;
   time (&seconds);
-  seconds += m_secondsDeltaZone;
+  seconds += calc_delta (seconds);
 #endif
 
 				// Return immediately if there is
@@ -735,18 +782,6 @@ void WTopLevel::render_line (GC gc, Pixmap pixmap, double theta,
 
 void WTopLevel::setup_time (void)
 {
-				// -- Determine timezone correction
-
-  time_t timeNow;
-  time (&timeNow);
-  struct tm tmGMT   = *gmtime (&timeNow);
-  struct tm tmLocal = *localtime (&timeNow);
-  tmGMT.tm_isdst = 0;		// We want the absolute difference
-  tmLocal.tm_isdst = 0;
-  m_secondsDeltaZone = mktime (&tmLocal) - mktime (&tmGMT);
-  //  fprintf (stderr, "deltaZone is %ld seconds or %d hours\n", 
-  //	   m_secondsDeltaZone, m_secondsDeltaZone/3600);
-
 				// -- Configure interval timer
   LSignal::accept (SIGALRM, signal_alarm, (void*) this, 0, 0);
   {
